// SPDX-License-Identifier: MIT
// SPDX-FileCopyrightText: 2025 Loïc Le Page <llepage@igalia.com>

#pragma once

#include <libcamera/libcamera.h>

class Camera final : public std::enable_shared_from_this<Camera>
{
  public:
    static std::shared_ptr<Camera> create(unsigned int captureWidth, unsigned int captureHeight) noexcept;
    ~Camera();

    Camera(Camera&&) = delete;
    Camera& operator=(Camera&&) = delete;
    Camera(const Camera&) = delete;
    Camera& operator=(const Camera&) = delete;

    void startCapturing() const noexcept;
    void stopCapturing() const noexcept;

    const std::vector<std::unique_ptr<libcamera::Request>>& getRequests() const noexcept
    {
        return m_requests;
    }

    class Frame final
    {
      public:
        ~Frame();

        Frame(Frame&&) = delete;
        Frame& operator=(Frame&&) = delete;
        Frame(const Frame&) = delete;
        Frame& operator=(const Frame&) = delete;

        uint64_t getCookie() const noexcept
        {
            return m_cookie;
        }

      private:
        friend class Camera;
        Frame(Camera* camera, libcamera::Request* request, uint64_t cookie)
            : m_camera(camera->weak_from_this()), m_request(request), m_cookie(cookie)
        {
        }

        std::weak_ptr<Camera> m_camera;
        libcamera::Request* m_request = nullptr;
        uint64_t m_cookie = 0;
    };

    std::unique_ptr<Frame> getNextFrame() noexcept;

  private:
    Camera() = default;

    void onRequestCompleted(libcamera::Request* request);
    std::mutex m_nextFrameMutex;
    std::unique_ptr<Frame> m_nextFrame;

    std::unique_ptr<libcamera::CameraManager> m_cameraManager;
    std::shared_ptr<libcamera::Camera> m_selectedCamera;
    std::unique_ptr<libcamera::FrameBufferAllocator> m_frameAllocator;
    std::vector<std::unique_ptr<libcamera::Request>> m_requests;
};
