// SPDX-License-Identifier: MIT
// SPDX-FileCopyrightText: 2025 Loïc Le Page <llepage@igalia.com>

#pragma once

#include "IRenderer.h"

#define GLFW_INCLUDE_ES3
#define GLFW_INCLUDE_GLEXT
#include <GLFW/glfw3.h>

#include <memory>

class AppWindow final
{
  public:
    static void terminate() noexcept
    {
        glfwTerminate();
    }

    static std::unique_ptr<AppWindow> create(int width, int height, const char* title,
                                             std::unique_ptr<IRenderer> renderer) noexcept;
    ~AppWindow();

    AppWindow(AppWindow&&) = delete;
    AppWindow& operator=(AppWindow&&) = delete;
    AppWindow(const AppWindow&) = delete;
    AppWindow& operator=(const AppWindow&) = delete;

    void runRenderLoop() const noexcept;

  private:
    AppWindow() = default;

    GLFWwindow* m_window = nullptr;
    std::unique_ptr<IRenderer> m_renderer;
};
