// SPDX-License-Identifier: MIT
// SPDX-FileCopyrightText: 2025 Loïc Le Page <llepage@igalia.com>

#pragma once

#include <GLES3/gl3.h>
#include <glm/glm.hpp>

#include <span>

struct Vertex
{
    glm::vec3 position;
    glm::vec3 normal;
    glm::vec2 texCoords;
};

class Mesh
{
  public:
    virtual ~Mesh();

    Mesh(Mesh&&) = delete;
    Mesh& operator=(Mesh&&) = delete;
    Mesh(const Mesh&) = delete;
    Mesh& operator=(const Mesh&) = delete;

    void draw() const noexcept;

  protected:
    Mesh() = default;
    bool initGL(std::span<const Vertex> vertices, std::span<const unsigned int> indices) noexcept;

  private:
    GLsizei m_indicesCount = 0;
    GLuint m_vao = 0;
    GLuint m_vbo = 0;
    GLuint m_ebo = 0;
};
