// SPDX-License-Identifier: MIT
// SPDX-FileCopyrightText: 2025 Loïc Le Page <llepage@igalia.com>

#pragma once

#include "Shader.h"

#include <glm/glm.hpp>

#include <memory>

class PhongShader final : public Shader
{
  public:
    static std::unique_ptr<PhongShader> create() noexcept;

    void setModelMatrix(const glm::mat4& matrix) const noexcept
    {
        glUniformMatrix4fv(m_modelMatrixLocation, 1, GL_FALSE, &matrix[0][0]);
    }

    void setViewMatrix(const glm::mat4& matrix) const noexcept
    {
        glUniformMatrix4fv(m_viewMatrixLocation, 1, GL_FALSE, &matrix[0][0]);
    }

    void setProjectionMatrix(const glm::mat4& matrix) const noexcept
    {
        glUniformMatrix4fv(m_projectionMatrixLocation, 1, GL_FALSE, &matrix[0][0]);
    }

    void setCameraAspectRatio(float ratio) const noexcept
    {
        glUniform1f(m_cameraAspectRatioLocation, ratio);
    }

    void setMaterialAmbient(const glm::vec3& color) const noexcept
    {
        glUniform3fv(m_materialAmbientLocation, 1, &color[0]);
    }

    void setMaterialDiffuse(const glm::vec3& color) const noexcept
    {
        glUniform3fv(m_materialDiffuseLocation, 1, &color[0]);
    }

    void setMaterialSpecular(const glm::vec3& color) const noexcept
    {
        glUniform3fv(m_materialSpecularLocation, 1, &color[0]);
    }

    void setMaterialShininess(float shininess) const noexcept
    {
        glUniform1f(m_materialShininessLocation, shininess);
    }

    void setCameraTextureMix(float mix) const noexcept
    {
        glUniform1f(m_cameraTextureMixLocation, glm::clamp(mix, 0.0f, 1.0f));
    }

    void setLightPosition(const glm::vec3& position) const noexcept
    {
        glUniform3fv(m_lightPositionLocation, 1, &position[0]);
    }

    void setLightColor(const glm::vec3& color) const noexcept
    {
        glUniform3fv(m_lightColorLocation, 1, &color[0]);
    }

    void setViewPosition(const glm::vec3& position) const noexcept
    {
        glUniform3fv(m_viewPositionLocation, 1, &position[0]);
    }

  private:
    PhongShader() = default;

    GLint m_modelMatrixLocation = -1;
    GLint m_viewMatrixLocation = -1;
    GLint m_projectionMatrixLocation = -1;
    GLint m_cameraAspectRatioLocation = -1;

    GLint m_materialAmbientLocation = -1;
    GLint m_materialDiffuseLocation = -1;
    GLint m_materialSpecularLocation = -1;
    GLint m_materialShininessLocation = -1;

    GLint m_cameraTextureMixLocation = -1;

    GLint m_lightPositionLocation = -1;
    GLint m_lightColorLocation = -1;

    GLint m_viewPositionLocation = -1;
};
