// SPDX-License-Identifier: MIT
// SPDX-FileCopyrightText: 2025 Loïc Le Page <llepage@igalia.com>

#include "Shader.h"

#include <cstring>
#include <iostream>

bool Shader::initGL(const char* vertexCode, const char* fragmentCode) noexcept
{
    if (!vertexCode || !*vertexCode || !fragmentCode || !*fragmentCode)
    {
        return false;
    }

    int vtxCodeLen = std::strlen(vertexCode);
    GLuint vtxShader = glCreateShader(GL_VERTEX_SHADER);
    glShaderSource(vtxShader, 1, &vertexCode, &vtxCodeLen);
    glCompileShader(vtxShader);

    int fragCodeLen = std::strlen(fragmentCode);
    GLuint fragShader = glCreateShader(GL_FRAGMENT_SHADER);
    glShaderSource(fragShader, 1, &fragmentCode, &fragCodeLen);
    glCompileShader(fragShader);

    m_program = glCreateProgram();
    glAttachShader(m_program, vtxShader);
    glAttachShader(m_program, fragShader);
    glLinkProgram(m_program);

    glDeleteShader(vtxShader);
    glDeleteShader(fragShader);

    GLint status = 0;
    glGetProgramiv(m_program, GL_LINK_STATUS, &status);
    if (!status)
    {
        GLsizei bufferSize = 0;
        glGetProgramiv(m_program, GL_INFO_LOG_LENGTH, &bufferSize);
        if (bufferSize > 0)
        {
            GLchar* buffer = new GLchar[bufferSize];
            glGetProgramInfoLog(m_program, bufferSize, nullptr, buffer);
            std::cerr << "Shader compilation error:" << std::endl << buffer << std::endl;
            delete[] buffer;
        }

        glDeleteProgram(m_program);
        m_program = 0;
        return false;
    }

    return true;
}

Shader::~Shader()
{
    if (m_program)
    {
        glDeleteProgram(m_program);
    }
}

void Shader::use() const noexcept
{
    glUseProgram(m_program);
}
